"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CSSStyleService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
// class StyleSheet implements IStyleSheet {}
class GlobalStyleSheet {
    constructor() {
        const style = document.createElement('style');
        document.body.appendChild(style);
        this.styleSheet = style.sheet;
    }
    insertSelector(selector, rule) {
        const index = this.styleSheet.insertRule(`${selector} { ${rule} }`, this.styleSheet.cssRules.length);
        return {
            index,
            dispose: () => {
                this.styleSheet.deleteRule(index);
            },
        };
    }
    deleteRule(index) {
        this.styleSheet.deleteRule(index);
    }
    removeClass(classname) {
        for (let i = this.styleSheet.rules.length - 1; i >= 0; i--) {
            if (this.styleSheet.rules[i].selectorText === '.' + classname) {
                this.styleSheet.removeRule(i);
            }
        }
    }
}
let CSSStyleService = class CSSStyleService {
    constructor() {
        this.globalStyleSheet = new GlobalStyleSheet();
    }
    acquire(key) {
        throw new Error('Method not implemented.');
    }
    addClass(classname, style) {
        const result = this.globalStyleSheet.insertSelector(`.${classname}`, createStyleCssText(style));
        return result;
    }
    deleteRule(index) {
        this.globalStyleSheet.deleteRule(index);
    }
    removeClass(classname) {
        this.globalStyleSheet.removeClass(classname);
    }
    _getOrCreateStyleSheet(key) { }
};
CSSStyleService = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], CSSStyleService);
exports.CSSStyleService = CSSStyleService;
function createStyleCssText(styles) {
    const texts = [];
    Object.keys(styles).forEach((key) => {
        if (styles[key]) {
            texts.push(toDashSplitForm(key) + ':' + styles[key]);
        }
    });
    return texts.join(';');
}
function toDashSplitForm(key) {
    return key
        .trim()
        .split(/(?=[A-Z])/)
        .join('-')
        .toLowerCase();
}
//# sourceMappingURL=style.service.js.map